@@ -38,6 +38,7 @@ type job = {
   job_stdin   : out_channel;
   job_stderr  : in_channel;
   job_buffer  : Buffer.t;
+  job_cleanup : (unit -> unit) option;
   mutable job_dying : bool;
 };;
 
@@ -73,7 +74,10 @@ let output_lines prefix oc buffer =
         try String.index_from u i '\n'
         with Not_found -> m
       in
-      output_line i j;
+      (* ignore trailing CR *)
+      let k = ref j in
+      while !k > (i : int) && u.[!k - 1] = '\r' do decr k done;
+      output_line i !k;
       loop (j + 1)
     else
       ()
@@ -134,10 +138,17 @@ let execute
   (*** add_job *)
   let add_job cmd rest result id =
     (*display begin fun oc -> fp oc "Job %a is %s\n%!" print_job_id id cmd; end;*)
-    let (stdout', stdin', stderr') = open_process_full cmd env in
+    let (stdout', stdin', stderr'), cleanup =
+      if Sys.win32
+      then
+        let args, cleanup = My_std.prepare_command_for_windows cmd in
+        open_process_args_full args.(0) args env, cleanup
+      else open_process_full cmd env, None in
     incr jobs_active;
-    set_nonblock (doi stdout');
-    set_nonblock (doi stderr');
+    if not Sys.win32 then begin
+      set_nonblock (doi stdout');
+      set_nonblock (doi stderr');
+    end;
     let job =
       { job_id          = id;
         job_command     = cmd;
@@ -147,7 +158,8 @@ let execute
         job_stdin       = stdin';
         job_stderr      = stderr';
         job_buffer      = Buffer.create 1024;
-        job_dying       = false }
+        job_dying       = false;
+        job_cleanup     = cleanup }
     in
     outputs := FDM.add (doi stdout') job (FDM.add (doi stderr') job !outputs);
     jobs := JS.add job !jobs;
@@ -245,14 +257,17 @@ let execute
       decr jobs_active;
 
       (* PR#5371: we would get EAGAIN below otherwise *)
-      clear_nonblock (doi job.job_stdout);
-      clear_nonblock (doi job.job_stderr);
+      if not Sys.win32 then begin
+        clear_nonblock (doi job.job_stdout);
+        clear_nonblock (doi job.job_stderr);
+      end;
 
       do_read ~loop:true (doi job.job_stdout) job;
       do_read ~loop:true (doi job.job_stderr) job;
       outputs := FDM.remove (doi job.job_stdout) (FDM.remove (doi job.job_stderr) !outputs);
       jobs := JS.remove job !jobs;
       let status = close_process_full (job.job_stdout, job.job_stdin, job.job_stderr) in
+      Option.iter (fun f -> f ()) job.job_cleanup;
 
       let shown = ref false in
 
