@@ -15,7 +15,7 @@ cd `dirname $0`
 set -e
 set -x
 CMDOTPS="" # -- command args
-BUILD="$OCB a.byte a.native proj.docdir/index.html -no-skip -classic-display $@"
+BUILD="$OCB a.byte a.native proj.docdir/index.html -classic-display $@"
 BUILD1="$BUILD $CMDOPTS"
 BUILD2="$BUILD -verbose 0 -nothing-should-be-rebuilt $CMDOPTS"
 rm -rf _build
