@@ -15,7 +15,7 @@ cd `dirname $0`
 set -e
 set -x
 CMDOTPS="" # -- command args
-BUILD="$OCB -I a -I b aa.byte aa.native -no-skip -classic-display $@"
+BUILD="$OCB -I a -I b aa.byte aa.native -classic-display $@"
 BUILD1="$BUILD $CMDOPTS"
 BUILD2="$BUILD -verbose 0 -nothing-should-be-rebuilt $CMDOPTS"
 rm -rf _build
