@@ -11,8 +11,17 @@
 (*                                                                     *)
 (***********************************************************************)
 
+#directory "../plugin-lib/";;
+#directory "../src/";;
+
+#load "../src/ocamlbuild_config.cmo"
+#load "../plugin-lib/ocamlbuildlib.cma";;
+
 open Format
 
+module My_std = Ocamlbuild_pack.My_std
+open Ocamlbuild_pack.Pathname.Operators
+
 external (|>) :  'a -> ('a -> 'b) -> 'b = "%revapply"
 
 let print_list ~sep f ppf = function
@@ -30,17 +39,18 @@ let print_string_list = print_list_com pp_print_string
 let print_string_list_com = print_list_com pp_print_string
 let print_string_list_blank = print_list_blank pp_print_string
 
-let exists filename =
-  try ignore(Unix.stat filename); true
-  with Unix.Unix_error ((Unix.ENOENT),_,_) -> false
+let exists filename = My_std.sys_file_exists filename
 
 let execute cmd =
   let ic = Unix.open_process_in cmd and lst = ref [] in
+  set_binary_mode_in ic false;
   try while true do lst := input_line ic :: !lst done; assert false
   with End_of_file ->
     let ret_code = Unix.close_process_in ic
     in ret_code, List.rev !lst
 
+let sys_command cmd = My_std.sys_command cmd
+
 let rm f =
   if exists f then
     ignore(Sys.command (Printf.sprintf "rm -r %s" f))
@@ -108,6 +118,7 @@ module Match = struct
   let x ?(atts=()) name ~output = X ((atts,name), (0,output))
 
   let match_with_fs ~root m =
+    My_std.reset_readdir_cache ();
     let rec visit ~exact ~successes ~errors path m =
       let string_of_path path = "./" ^ String.concat "/" (List.rev path) in
       let file name = string_of_path (name :: path) in
@@ -381,6 +392,10 @@ module Tree = struct
 
 end
 
+let a = Ocamlbuild_config.a
+let o = Ocamlbuild_config.o
+let so = Ocamlbuild_config.so
+
 type content = string
 type filename = string
 type run = filename * content
@@ -395,6 +410,7 @@ type test = { name     : string
             ; options  : Option.t list
             ; targets  : string * string list
             ; pre_cmd  : string option
+            ; post_cmd : string option
             ; failing_msg : string option
             ; run      : run list }
 
@@ -403,7 +419,7 @@ let tests = ref []
 let test name
     ~description
     ?requirements
-    ?(options=[]) ?(run=[]) ?pre_cmd ?failing_msg
+    ?(options=[]) ?(run=[]) ?pre_cmd ?post_cmd ?failing_msg
     ?(tree=[])
     ?(matching=[])
     ~targets ()
@@ -417,6 +433,7 @@ let test name
     options;
     targets;
     pre_cmd;
+    post_cmd;
     failing_msg;
     run;
   }]
@@ -435,19 +452,50 @@ let print_colored header_color header name body_color body =
     (color_code header_color) header name
     (color_code body_color) body
 
+let rec mkdir_p dir =
+  if not (Sys.file_exists dir) then begin
+    mkdir_p (Filename.dirname dir);
+    Unix.mkdir dir 0o750;
+  end
+
 let run ~root =
   let dir = Sys.getcwd () in
   let root = dir ^ "/" ^ root in
   rm root;
   Unix.mkdir root 0o750;
+  let install_lib_dir = Filename.concat root "install-lib" in
+  let install_bin_dir = Filename.concat root "install-bin" in
 
   let test_tree = Filename.dirname (Sys.argv.(0)) in
   let test_tree = if test_tree = "." then Sys.getcwd () else test_tree in
   let build_tree = Filename.dirname test_tree in
-  let ocamlbuild = Filename.concat build_tree "ocamlbuild.byte" in
+
+  let copy l dest =
+    mkdir_p dest;
+    List.iter (fun f -> ignore(Sys.command (Printf.sprintf "cp %s/%s %s" build_tree f dest))) l
+  in
+  let ocamlbuild = Printf.sprintf "%s/ocamlbuild.byte" install_bin_dir in
+  copy
+    [ "plugin-lib/ocamlbuildlib.cma";
+      "plugin-lib/ocamlbuildlib.cmxa";
+      "plugin-lib/ocamlbuildlib" -.- a;
+      "bin/ocamlbuild.cmo";
+      "bin/ocamlbuild.cmx";
+      "bin/ocamlbuild" -.- o;
+      "src/ocamlbuild_pack.cmi";
+      "src/ocamlbuild_pack.cmx";
+      "src/ocamlbuild_pack" -.- o;
+      "plugin-lib/ocamlbuild_plugin.cmi";
+      "plugin-lib/ocamlbuild_plugin.cmx";
+      "plugin-lib/ocamlbuild_plugin" -.- o ]
+    install_lib_dir;
+  copy
+    [ "ocamlbuild.byte";
+      "ocamlbuild.native" ]
+    install_bin_dir;
   let testsuite_opts = [
-    (`install_bin_dir build_tree);
-    (`install_lib_dir (Filename.concat build_tree "src"));
+    (`install_bin_dir install_bin_dir);
+    (`install_lib_dir install_lib_dir);
   ] in
 
   let verbose =
@@ -478,6 +526,7 @@ let run ~root =
       ; targets
       ; failing_msg
       ; pre_cmd
+      ; post_cmd
       ; run } =
 
     let full_name = root ^ "/" ^ name in
@@ -495,7 +544,7 @@ let run ~root =
 
         (match pre_cmd with
           | None -> ()
-          | Some str -> ignore(Sys.command str));
+          | Some str -> ignore(sys_command str));
 
         let log_name = full_name ^ ".log" in
 
@@ -503,7 +552,11 @@ let run ~root =
         let allow_failure = failing_msg <> None in
 
         let open Unix in
-
+        let post_cmd () =
+          match post_cmd with
+          | None -> true
+          | Some str -> sys_command str = 0
+        in
         match execute cmd with
           | WEXITED n,lines
           | WSIGNALED n,lines
@@ -532,8 +585,13 @@ let run ~root =
                   (* filter out -classic-display output *)
                   List.filter (fun s -> not (starts_with_plus s)) lines in
                 let msg = String.concat "\n" lines in
-                if failing_msg = msg then
-                  print_colored `Green "PASSED" name `Cyan description
+                if failing_msg = msg  then
+                  if post_cmd ()
+                  then print_colored `Green "PASSED" name `Cyan description
+                  else begin
+                    print_colored `Red "FAILED" name `Yellow "post command failed";
+                    failed := true
+                  end
                 else begin
                   print_colored `Red "FAILED" name `Yellow
                     ((Printf.sprintf "Failure with not matching message:\n\
@@ -546,7 +604,13 @@ let run ~root =
               List.concat
                 (List.map (Match.match_with_fs ~root:full_name) matching) in
             begin if errors == [] then
-                print_colored `Green "PASSED" name `Cyan description
+                if post_cmd ()
+                then
+                  print_colored `Green "PASSED" name `Cyan description
+                else begin
+                    print_colored `Red "FAILED" name `Yellow "post command failed";
+                    failed := true
+                  end
               else begin
                 if verbose then begin
                   print_colored `Red "FAILED" name `Yellow
